/*******************************************************************************
* File Name: ETH_A_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ETH_A.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (ETH_A_RX_ENABLED || ETH_A_HD_ENABLED) && \
     (ETH_A_RXBUFFERSIZE > ETH_A_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: ETH_A_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  ETH_A_rxBuffer - RAM buffer pointer for save received data.
    *  ETH_A_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  ETH_A_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  ETH_A_rxBufferOverflow - software overflow flag. Set to one
    *     when ETH_A_rxBufferWrite index overtakes
    *     ETH_A_rxBufferRead index.
    *  ETH_A_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when ETH_A_rxBufferWrite is equal to
    *    ETH_A_rxBufferRead
    *  ETH_A_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  ETH_A_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(ETH_A_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START ETH_A_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = ETH_A_RXSTATUS_REG;

        if((readData & (ETH_A_RX_STS_BREAK | ETH_A_RX_STS_PAR_ERROR |
                        ETH_A_RX_STS_STOP_ERROR | ETH_A_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START ETH_A_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & ETH_A_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (ETH_A_RXHW_ADDRESS_ENABLED)
                if(ETH_A_rxAddressMode == (uint8)ETH_A__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & ETH_A_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & ETH_A_RX_STS_ADDR_MATCH) != 0u)
                        {
                            ETH_A_rxAddressDetected = 1u;
                        }
                        else
                        {
                            ETH_A_rxAddressDetected = 0u;
                        }
                    }

                    readData = ETH_A_RXDATA_REG;
                    if(ETH_A_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        ETH_A_rxBuffer[ETH_A_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    ETH_A_rxBuffer[ETH_A_rxBufferWrite] = ETH_A_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                ETH_A_rxBuffer[ETH_A_rxBufferWrite] = ETH_A_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(ETH_A_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    ETH_A_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                ETH_A_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(ETH_A_rxBufferWrite >= ETH_A_RXBUFFERSIZE)
                {
                    ETH_A_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(ETH_A_rxBufferWrite == ETH_A_rxBufferRead)
                {
                    ETH_A_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(ETH_A_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        ETH_A_RXSTATUS_MASK_REG  &= (uint8)~ETH_A_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(ETH_A_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End ETH_A_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = ETH_A_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START ETH_A_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End ETH_A_RX_ENABLED && (ETH_A_RXBUFFERSIZE > ETH_A_FIFO_LENGTH) */


#if(ETH_A_TX_ENABLED && (ETH_A_TXBUFFERSIZE > ETH_A_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: ETH_A_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  ETH_A_txBuffer - RAM buffer pointer for transmit data from.
    *  ETH_A_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  ETH_A_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(ETH_A_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START ETH_A_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((ETH_A_txBufferRead != ETH_A_txBufferWrite) &&
             ((ETH_A_TXSTATUS_REG & ETH_A_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(ETH_A_txBufferRead >= ETH_A_TXBUFFERSIZE)
            {
                ETH_A_txBufferRead = 0u;
            }

            ETH_A_TXDATA_REG = ETH_A_txBuffer[ETH_A_txBufferRead];

            /* Set next pointer. */
            ETH_A_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START ETH_A_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End ETH_A_TX_ENABLED && (ETH_A_TXBUFFERSIZE > ETH_A_FIFO_LENGTH) */


/* [] END OF FILE */
