/*******************************************************************************
* File Name: ETH_B_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ETH_B.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (ETH_B_RX_ENABLED || ETH_B_HD_ENABLED) && \
     (ETH_B_RXBUFFERSIZE > ETH_B_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: ETH_B_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  ETH_B_rxBuffer - RAM buffer pointer for save received data.
    *  ETH_B_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  ETH_B_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  ETH_B_rxBufferOverflow - software overflow flag. Set to one
    *     when ETH_B_rxBufferWrite index overtakes
    *     ETH_B_rxBufferRead index.
    *  ETH_B_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when ETH_B_rxBufferWrite is equal to
    *    ETH_B_rxBufferRead
    *  ETH_B_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  ETH_B_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(ETH_B_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START ETH_B_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = ETH_B_RXSTATUS_REG;

        if((readData & (ETH_B_RX_STS_BREAK | ETH_B_RX_STS_PAR_ERROR |
                        ETH_B_RX_STS_STOP_ERROR | ETH_B_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START ETH_B_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & ETH_B_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (ETH_B_RXHW_ADDRESS_ENABLED)
                if(ETH_B_rxAddressMode == (uint8)ETH_B__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & ETH_B_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & ETH_B_RX_STS_ADDR_MATCH) != 0u)
                        {
                            ETH_B_rxAddressDetected = 1u;
                        }
                        else
                        {
                            ETH_B_rxAddressDetected = 0u;
                        }
                    }

                    readData = ETH_B_RXDATA_REG;
                    if(ETH_B_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        ETH_B_rxBuffer[ETH_B_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    ETH_B_rxBuffer[ETH_B_rxBufferWrite] = ETH_B_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                ETH_B_rxBuffer[ETH_B_rxBufferWrite] = ETH_B_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(ETH_B_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    ETH_B_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                ETH_B_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(ETH_B_rxBufferWrite >= ETH_B_RXBUFFERSIZE)
                {
                    ETH_B_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(ETH_B_rxBufferWrite == ETH_B_rxBufferRead)
                {
                    ETH_B_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(ETH_B_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        ETH_B_RXSTATUS_MASK_REG  &= (uint8)~ETH_B_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(ETH_B_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End ETH_B_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = ETH_B_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START ETH_B_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End ETH_B_RX_ENABLED && (ETH_B_RXBUFFERSIZE > ETH_B_FIFO_LENGTH) */


#if(ETH_B_TX_ENABLED && (ETH_B_TXBUFFERSIZE > ETH_B_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: ETH_B_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  ETH_B_txBuffer - RAM buffer pointer for transmit data from.
    *  ETH_B_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  ETH_B_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(ETH_B_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START ETH_B_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((ETH_B_txBufferRead != ETH_B_txBufferWrite) &&
             ((ETH_B_TXSTATUS_REG & ETH_B_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(ETH_B_txBufferRead >= ETH_B_TXBUFFERSIZE)
            {
                ETH_B_txBufferRead = 0u;
            }

            ETH_B_TXDATA_REG = ETH_B_txBuffer[ETH_B_txBufferRead];

            /* Set next pointer. */
            ETH_B_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START ETH_B_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End ETH_B_TX_ENABLED && (ETH_B_TXBUFFERSIZE > ETH_B_FIFO_LENGTH) */


/* [] END OF FILE */
