/*******************************************************************************
* File Name: PL2_RS485_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "PL2_RS485.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (PL2_RS485_RX_ENABLED || PL2_RS485_HD_ENABLED) && \
     (PL2_RS485_RXBUFFERSIZE > PL2_RS485_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: PL2_RS485_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  PL2_RS485_rxBuffer - RAM buffer pointer for save received data.
    *  PL2_RS485_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  PL2_RS485_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  PL2_RS485_rxBufferOverflow - software overflow flag. Set to one
    *     when PL2_RS485_rxBufferWrite index overtakes
    *     PL2_RS485_rxBufferRead index.
    *  PL2_RS485_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when PL2_RS485_rxBufferWrite is equal to
    *    PL2_RS485_rxBufferRead
    *  PL2_RS485_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  PL2_RS485_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(PL2_RS485_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START PL2_RS485_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = PL2_RS485_RXSTATUS_REG;

        if((readData & (PL2_RS485_RX_STS_BREAK | PL2_RS485_RX_STS_PAR_ERROR |
                        PL2_RS485_RX_STS_STOP_ERROR | PL2_RS485_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START PL2_RS485_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & PL2_RS485_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (PL2_RS485_RXHW_ADDRESS_ENABLED)
                if(PL2_RS485_rxAddressMode == (uint8)PL2_RS485__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & PL2_RS485_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & PL2_RS485_RX_STS_ADDR_MATCH) != 0u)
                        {
                            PL2_RS485_rxAddressDetected = 1u;
                        }
                        else
                        {
                            PL2_RS485_rxAddressDetected = 0u;
                        }
                    }

                    readData = PL2_RS485_RXDATA_REG;
                    if(PL2_RS485_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        PL2_RS485_rxBuffer[PL2_RS485_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    PL2_RS485_rxBuffer[PL2_RS485_rxBufferWrite] = PL2_RS485_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                PL2_RS485_rxBuffer[PL2_RS485_rxBufferWrite] = PL2_RS485_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(PL2_RS485_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    PL2_RS485_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                PL2_RS485_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(PL2_RS485_rxBufferWrite >= PL2_RS485_RXBUFFERSIZE)
                {
                    PL2_RS485_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(PL2_RS485_rxBufferWrite == PL2_RS485_rxBufferRead)
                {
                    PL2_RS485_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(PL2_RS485_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        PL2_RS485_RXSTATUS_MASK_REG  &= (uint8)~PL2_RS485_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(PL2_RS485_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End PL2_RS485_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = PL2_RS485_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START PL2_RS485_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End PL2_RS485_RX_ENABLED && (PL2_RS485_RXBUFFERSIZE > PL2_RS485_FIFO_LENGTH) */


#if(PL2_RS485_TX_ENABLED && (PL2_RS485_TXBUFFERSIZE > PL2_RS485_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: PL2_RS485_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  PL2_RS485_txBuffer - RAM buffer pointer for transmit data from.
    *  PL2_RS485_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  PL2_RS485_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(PL2_RS485_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START PL2_RS485_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((PL2_RS485_txBufferRead != PL2_RS485_txBufferWrite) &&
             ((PL2_RS485_TXSTATUS_REG & PL2_RS485_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(PL2_RS485_txBufferRead >= PL2_RS485_TXBUFFERSIZE)
            {
                PL2_RS485_txBufferRead = 0u;
            }

            PL2_RS485_TXDATA_REG = PL2_RS485_txBuffer[PL2_RS485_txBufferRead];

            /* Set next pointer. */
            PL2_RS485_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START PL2_RS485_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End PL2_RS485_TX_ENABLED && (PL2_RS485_TXBUFFERSIZE > PL2_RS485_FIFO_LENGTH) */


/* [] END OF FILE */
