/*******************************************************************************
* File Name: RS232.c
* Version 2.30
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "RS232.h"
#include "CyLib.h"
#if(RS232_INTERNAL_CLOCK_USED)
    #include "RS232_IntClock.h"
#endif /* End RS232_INTERNAL_CLOCK_USED */


/***************************************
* Global data allocation
***************************************/

uint8 RS232_initVar = 0u;
#if( RS232_TX_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH))
    volatile uint8 RS232_txBuffer[RS232_TXBUFFERSIZE];
    volatile uint8 RS232_txBufferRead = 0u;
    uint8 RS232_txBufferWrite = 0u;
#endif /* End RS232_TX_ENABLED */
#if( ( RS232_RX_ENABLED || RS232_HD_ENABLED ) && \
     (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH) )
    volatile uint8 RS232_rxBuffer[RS232_RXBUFFERSIZE];
    volatile uint8 RS232_rxBufferRead = 0u;
    volatile uint8 RS232_rxBufferWrite = 0u;
    volatile uint8 RS232_rxBufferLoopDetect = 0u;
    volatile uint8 RS232_rxBufferOverflow = 0u;
    #if (RS232_RXHW_ADDRESS_ENABLED)
        volatile uint8 RS232_rxAddressMode = RS232_RXADDRESSMODE;
        volatile uint8 RS232_rxAddressDetected = 0u;
    #endif /* End EnableHWAddress */
#endif /* End RS232_RX_ENABLED */


/*******************************************************************************
* Function Name: RS232_Start
********************************************************************************
*
* Summary:
*  Initialize and Enable the UART component.
*  Enable the clock input to enable operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The RS232_intiVar variable is used to indicate initial
*  configuration of this component. The variable is initialized to zero (0u)
*  and set to one (1u) the first time UART_Start() is called. This allows for
*  component initialization without re-initialization in all subsequent calls
*  to the RS232_Start() routine.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void RS232_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(RS232_initVar == 0u)
    {
        RS232_Init();
        RS232_initVar = 1u;
    }
    RS232_Enable();
}


/*******************************************************************************
* Function Name: RS232_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the
*  customizer of the component placed onto schematic. Usually called in
*  RS232_Start().
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void RS232_Init(void) 
{
    #if(RS232_RX_ENABLED || RS232_HD_ENABLED)

        #if(RS232_RX_INTERRUPT_ENABLED && (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH))
            /* Set the RX Interrupt. */
            (void)CyIntSetVector(RS232_RX_VECT_NUM, &RS232_RXISR);
            CyIntSetPriority(RS232_RX_VECT_NUM, RS232_RX_PRIOR_NUM);
        #endif /* End RS232_RX_INTERRUPT_ENABLED */

        #if (RS232_RXHW_ADDRESS_ENABLED)
            RS232_SetRxAddressMode(RS232_RXAddressMode);
            RS232_SetRxAddress1(RS232_RXHWADDRESS1);
            RS232_SetRxAddress2(RS232_RXHWADDRESS2);
        #endif /* End RS232_RXHW_ADDRESS_ENABLED */

        /* Init Count7 period */
        RS232_RXBITCTR_PERIOD_REG = RS232_RXBITCTR_INIT;
        /* Configure the Initial RX interrupt mask */
        RS232_RXSTATUS_MASK_REG  = RS232_INIT_RX_INTERRUPTS_MASK;
    #endif /* End RS232_RX_ENABLED || RS232_HD_ENABLED*/

    #if(RS232_TX_ENABLED)
        #if(RS232_TX_INTERRUPT_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH))
            /* Set the TX Interrupt. */
            (void)CyIntSetVector(RS232_TX_VECT_NUM, &RS232_TXISR);
            CyIntSetPriority(RS232_TX_VECT_NUM, RS232_TX_PRIOR_NUM);
        #endif /* End RS232_TX_INTERRUPT_ENABLED */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if(RS232_TXCLKGEN_DP)
            RS232_TXBITCLKGEN_CTR_REG = RS232_BIT_CENTER;
            RS232_TXBITCLKTX_COMPLETE_REG = (RS232_NUMBER_OF_DATA_BITS +
                        RS232_NUMBER_OF_START_BIT) * RS232_OVER_SAMPLE_COUNT;
        #else
            RS232_TXBITCTR_PERIOD_REG = ((RS232_NUMBER_OF_DATA_BITS +
                        RS232_NUMBER_OF_START_BIT) * RS232_OVER_SAMPLE_8) - 1u;
        #endif /* End RS232_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if(RS232_TX_INTERRUPT_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH))
            RS232_TXSTATUS_MASK_REG = RS232_TX_STS_FIFO_EMPTY;
        #else
            RS232_TXSTATUS_MASK_REG = RS232_INIT_TX_INTERRUPTS_MASK;
        #endif /*End RS232_TX_INTERRUPT_ENABLED*/

    #endif /* End RS232_TX_ENABLED */

    #if(RS232_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        RS232_WriteControlRegister( \
            (RS232_ReadControlRegister() & (uint8)~RS232_CTRL_PARITY_TYPE_MASK) | \
            (uint8)(RS232_PARITY_TYPE << RS232_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End RS232_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: RS232_Enable
********************************************************************************
*
* Summary:
*  Enables the UART block operation
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  RS232_rxAddressDetected - set to initial state (0).
*
*******************************************************************************/
void RS232_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    #if(RS232_RX_ENABLED || RS232_HD_ENABLED)
        /*RX Counter (Count7) Enable */
        RS232_RXBITCTR_CONTROL_REG |= RS232_CNTR_ENABLE;
        /* Enable the RX Interrupt. */
        RS232_RXSTATUS_ACTL_REG  |= RS232_INT_ENABLE;
        #if(RS232_RX_INTERRUPT_ENABLED && (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH))
            CyIntEnable(RS232_RX_VECT_NUM);
            #if (RS232_RXHW_ADDRESS_ENABLED)
                RS232_rxAddressDetected = 0u;
            #endif /* End RS232_RXHW_ADDRESS_ENABLED */
        #endif /* End RS232_RX_INTERRUPT_ENABLED */
    #endif /* End RS232_RX_ENABLED || RS232_HD_ENABLED*/

    #if(RS232_TX_ENABLED)
        /*TX Counter (DP/Count7) Enable */
        #if(!RS232_TXCLKGEN_DP)
            RS232_TXBITCTR_CONTROL_REG |= RS232_CNTR_ENABLE;
        #endif /* End RS232_TXCLKGEN_DP */
        /* Enable the TX Interrupt. */
        RS232_TXSTATUS_ACTL_REG |= RS232_INT_ENABLE;
        #if(RS232_TX_INTERRUPT_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH))
            CyIntEnable(RS232_TX_VECT_NUM);
        #endif /* End RS232_TX_INTERRUPT_ENABLED*/
     #endif /* End RS232_TX_ENABLED */

    #if(RS232_INTERNAL_CLOCK_USED)
        /* Enable the clock. */
        RS232_IntClock_Start();
    #endif /* End RS232_INTERNAL_CLOCK_USED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: RS232_Stop
********************************************************************************
*
* Summary:
*  Disable the UART component
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void RS232_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /* Write Bit Counter Disable */
    #if(RS232_RX_ENABLED || RS232_HD_ENABLED)
        RS232_RXBITCTR_CONTROL_REG &= (uint8)~RS232_CNTR_ENABLE;
    #endif /* End RS232_RX_ENABLED */

    #if(RS232_TX_ENABLED)
        #if(!RS232_TXCLKGEN_DP)
            RS232_TXBITCTR_CONTROL_REG &= (uint8)~RS232_CNTR_ENABLE;
        #endif /* End RS232_TXCLKGEN_DP */
    #endif /* RS232_TX_ENABLED */

    #if(RS232_INTERNAL_CLOCK_USED)
        /* Disable the clock. */
        RS232_IntClock_Stop();
    #endif /* End RS232_INTERNAL_CLOCK_USED */

    /* Disable internal interrupt component */
    #if(RS232_RX_ENABLED || RS232_HD_ENABLED)
        RS232_RXSTATUS_ACTL_REG  &= (uint8)~RS232_INT_ENABLE;
        #if(RS232_RX_INTERRUPT_ENABLED && (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH))
            RS232_DisableRxInt();
        #endif /* End RS232_RX_INTERRUPT_ENABLED */
    #endif /* End RS232_RX_ENABLED */

    #if(RS232_TX_ENABLED)
        RS232_TXSTATUS_ACTL_REG &= (uint8)~RS232_INT_ENABLE;
        #if(RS232_TX_INTERRUPT_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH))
            RS232_DisableTxInt();
        #endif /* End RS232_TX_INTERRUPT_ENABLED */
    #endif /* End RS232_TX_ENABLED */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: RS232_ReadControlRegister
********************************************************************************
*
* Summary:
*  Read the current state of the control register
*
* Parameters:
*  None.
*
* Return:
*  Current state of the control register.
*
*******************************************************************************/
uint8 RS232_ReadControlRegister(void) 
{
    #if( RS232_CONTROL_REG_REMOVED )
        return(0u);
    #else
        return(RS232_CONTROL_REG);
    #endif /* End RS232_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: RS232_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  RS232_WriteControlRegister(uint8 control) 
{
    #if( RS232_CONTROL_REG_REMOVED )
        if(control != 0u) { }      /* release compiler warning */
    #else
       RS232_CONTROL_REG = control;
    #endif /* End RS232_CONTROL_REG_REMOVED */
}


#if(RS232_RX_ENABLED || RS232_HD_ENABLED)

    #if(RS232_RX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: RS232_EnableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void RS232_EnableRxInt(void) 
        {
            CyIntEnable(RS232_RX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: RS232_DisableRxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable RX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void RS232_DisableRxInt(void) 
        {
            CyIntDisable(RS232_RX_VECT_NUM);
        }

    #endif /* RS232_RX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: RS232_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  IntSrc:  An or'd combination of the desired status bit masks (defined in
    *           the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void RS232_SetRxInterruptMode(uint8 intSrc) 
    {
        RS232_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: RS232_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns data in RX Data register without checking status register to
    *  determine if data is valid
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  RS232_rxBuffer - RAM buffer pointer for save received data.
    *  RS232_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  RS232_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  RS232_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 RS232_ReadRxData(void) 
    {
        uint8 rxData;

        #if(RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_DisableRxInt();
            #endif /* RS232_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = RS232_rxBufferRead;
            loc_rxBufferWrite = RS232_rxBufferWrite;

            if( (RS232_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = RS232_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;

                if(loc_rxBufferRead >= RS232_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                RS232_rxBufferRead = loc_rxBufferRead;

                if(RS232_rxBufferLoopDetect != 0u )
                {
                    RS232_rxBufferLoopDetect = 0u;
                    #if( (RS232_RX_INTERRUPT_ENABLED) && (RS232_FLOW_CONTROL != 0u) && \
                         (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( RS232_HD_ENABLED )
                            if((RS232_CONTROL_REG & RS232_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only in RX
                                *  configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                RS232_RXSTATUS_MASK_REG  |= RS232_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            RS232_RXSTATUS_MASK_REG  |= RS232_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end RS232_HD_ENABLED */
                    #endif /* RS232_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }
            }
            else
            {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
                rxData = RS232_RXDATA_REG;
            }

            /* Enable Rx interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_EnableRxInt();
            #endif /* End RS232_RX_INTERRUPT_ENABLED */

        #else /* RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

            /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit*/
            rxData = RS232_RXDATA_REG;

        #endif /* RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: RS232_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the current state of the status register
    *  And detect software buffer overflow.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Global Variables:
    *  RS232_rxBufferOverflow - used to indicate overload condition.
    *   It set to one in RX interrupt when there isn?t free space in
    *   RS232_rxBufferRead to write new data. This condition returned
    *   and cleared to zero by this API as an
    *   RS232_RX_STS_SOFT_BUFF_OVER bit along with RX Status register
    *   bits.
    *
    *******************************************************************************/
    uint8 RS232_ReadRxStatus(void) 
    {
        uint8 status;

        status = RS232_RXSTATUS_REG & RS232_RX_HW_MASK;

        #if(RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH)
            if( RS232_rxBufferOverflow != 0u )
            {
                status |= RS232_RX_STS_SOFT_BUFF_OVER;
                RS232_rxBufferOverflow = 0u;
            }
        #endif /* RS232_RXBUFFERSIZE */

        return(status);
    }


    /*******************************************************************************
    * Function Name: RS232_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, if data is not available or an error
    *  condition exists, zero is returned; otherwise, character is read and
    *  returned.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  RS232_rxBuffer - RAM buffer pointer for save received data.
    *  RS232_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  RS232_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  RS232_rxBufferLoopDetect - creared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 RS232_GetChar(void) 
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

        #if(RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH)
            uint8 loc_rxBufferRead;
            uint8 loc_rxBufferWrite;
            /* Protect variables that could change on interrupt. */
            /* Disable Rx interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_DisableRxInt();
            #endif /* RS232_RX_INTERRUPT_ENABLED */
            loc_rxBufferRead = RS232_rxBufferRead;
            loc_rxBufferWrite = RS232_rxBufferWrite;

            if( (RS232_rxBufferLoopDetect != 0u) || (loc_rxBufferRead != loc_rxBufferWrite) )
            {
                rxData = RS232_rxBuffer[loc_rxBufferRead];
                loc_rxBufferRead++;
                if(loc_rxBufferRead >= RS232_RXBUFFERSIZE)
                {
                    loc_rxBufferRead = 0u;
                }
                /* Update the real pointer */
                RS232_rxBufferRead = loc_rxBufferRead;

                if(RS232_rxBufferLoopDetect > 0u )
                {
                    RS232_rxBufferLoopDetect = 0u;
                    #if( (RS232_RX_INTERRUPT_ENABLED) && (RS232_FLOW_CONTROL != 0u) )
                        /* When Hardware Flow Control selected - return RX mask */
                        #if( RS232_HD_ENABLED )
                            if((RS232_CONTROL_REG & RS232_CTRL_HD_SEND) == 0u)
                            {   /* In Half duplex mode return RX mask only if
                                *  RX configuration set, otherwise
                                *  mask will be returned in LoadRxConfig() API.
                                */
                                RS232_RXSTATUS_MASK_REG  |= RS232_RX_STS_FIFO_NOTEMPTY;
                            }
                        #else
                            RS232_RXSTATUS_MASK_REG  |= RS232_RX_STS_FIFO_NOTEMPTY;
                        #endif /* end RS232_HD_ENABLED */
                    #endif /* RS232_RX_INTERRUPT_ENABLED and Hardware flow control*/
                }

            }
            else
            {   rxStatus = RS232_RXSTATUS_REG;
                if((rxStatus & RS232_RX_STS_FIFO_NOTEMPTY) != 0u)
                {   /* Read received data from FIFO*/
                    rxData = RS232_RXDATA_REG;
                    /*Check status on error*/
                    if((rxStatus & (RS232_RX_STS_BREAK | RS232_RX_STS_PAR_ERROR |
                                   RS232_RX_STS_STOP_ERROR | RS232_RX_STS_OVERRUN)) != 0u)
                    {
                        rxData = 0u;
                    }
                }
            }

            /* Enable Rx interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_EnableRxInt();
            #endif /* RS232_RX_INTERRUPT_ENABLED */

        #else /* RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

            rxStatus =RS232_RXSTATUS_REG;
            if((rxStatus & RS232_RX_STS_FIFO_NOTEMPTY) != 0u)
            {   /* Read received data from FIFO*/
                rxData = RS232_RXDATA_REG;
                /*Check status on error*/
                if((rxStatus & (RS232_RX_STS_BREAK | RS232_RX_STS_PAR_ERROR |
                               RS232_RX_STS_STOP_ERROR | RS232_RX_STS_OVERRUN)) != 0u)
                {
                    rxData = 0u;
                }
            }
        #endif /* RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: RS232_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Grab the next available byte of data from the recieve FIFO
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains Status Register and LSB contains UART RX data
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 RS232_GetByte(void) 
    {
        return ( ((uint16)RS232_ReadRxStatus() << 8u) | RS232_ReadRxData() );
    }


    /*******************************************************************************
    * Function Name: RS232_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of bytes left in the RX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Integer count of the number of bytes left
    *  in the RX buffer
    *
    * Global Variables:
    *  RS232_rxBufferWrite - used to calculate left bytes.
    *  RS232_rxBufferRead - used to calculate left bytes.
    *  RS232_rxBufferLoopDetect - checked to decide left bytes amount.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 RS232_GetRxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH)

            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_DisableRxInt();
            #endif /* RS232_RX_INTERRUPT_ENABLED */

            if(RS232_rxBufferRead == RS232_rxBufferWrite)
            {
                if(RS232_rxBufferLoopDetect > 0u)
                {
                    size = RS232_RXBUFFERSIZE;
                }
                else
                {
                    size = 0u;
                }
            }
            else if(RS232_rxBufferRead < RS232_rxBufferWrite)
            {
                size = (RS232_rxBufferWrite - RS232_rxBufferRead);
            }
            else
            {
                size = (RS232_RXBUFFERSIZE - RS232_rxBufferRead) + RS232_rxBufferWrite;
            }

            /* Enable Rx interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_EnableRxInt();
            #endif /* End RS232_RX_INTERRUPT_ENABLED */

        #else /* RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

            /* We can only know if there is data in the fifo. */
            size = ((RS232_RXSTATUS_REG & RS232_RX_STS_FIFO_NOTEMPTY) != 0u) ? 1u : 0u;

        #endif /* End RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: RS232_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the RX RAM buffer by setting the read and write pointers both to zero.
    *  Clears hardware RX FIFO.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_rxBufferWrite - cleared to zero.
    *  RS232_rxBufferRead - cleared to zero.
    *  RS232_rxBufferLoopDetect - cleared to zero.
    *  RS232_rxBufferOverflow - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM or FIFO buffer will be lost.
    *******************************************************************************/
    void RS232_ClearRxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* clear the HW FIFO */
        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        RS232_RXDATA_AUX_CTL_REG |=  RS232_RX_FIFO_CLR;
        RS232_RXDATA_AUX_CTL_REG &= (uint8)~RS232_RX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH)
            /* Disable Rx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_DisableRxInt();
            #endif /* End RS232_RX_INTERRUPT_ENABLED */

            RS232_rxBufferRead = 0u;
            RS232_rxBufferWrite = 0u;
            RS232_rxBufferLoopDetect = 0u;
            RS232_rxBufferOverflow = 0u;

            /* Enable Rx interrupt. */
            #if(RS232_RX_INTERRUPT_ENABLED)
                RS232_EnableRxInt();
            #endif /* End RS232_RX_INTERRUPT_ENABLED */
        #endif /* End RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH */

    }


    /*******************************************************************************
    * Function Name: RS232_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the receive addressing mode
    *
    * Parameters:
    *  addressMode: Enumerated value indicating the mode of RX addressing
    *  RS232__B_UART__AM_SW_BYTE_BYTE -  Software Byte-by-Byte address
    *                                               detection
    *  RS232__B_UART__AM_SW_DETECT_TO_BUFFER - Software Detect to Buffer
    *                                               address detection
    *  RS232__B_UART__AM_HW_BYTE_BY_BYTE - Hardware Byte-by-Byte address
    *                                               detection
    *  RS232__B_UART__AM_HW_DETECT_TO_BUFFER - Hardware Detect to Buffer
    *                                               address detection
    *  RS232__B_UART__AM_NONE - No address detection
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_rxAddressMode - the parameter stored in this variable for
    *   the farther usage in RX ISR.
    *  RS232_rxAddressDetected - set to initial state (0).
    *
    *******************************************************************************/
    void RS232_SetRxAddressMode(uint8 addressMode)
                                                        
    {
        #if(RS232_RXHW_ADDRESS_ENABLED)
            #if(RS232_CONTROL_REG_REMOVED)
                if(addressMode != 0u) { }     /* release compiler warning */
            #else /* RS232_CONTROL_REG_REMOVED */
                uint8 tmpCtrl;
                tmpCtrl = RS232_CONTROL_REG & (uint8)~RS232_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= (uint8)(addressMode << RS232_CTRL_RXADDR_MODE0_SHIFT);
                RS232_CONTROL_REG = tmpCtrl;
                #if(RS232_RX_INTERRUPT_ENABLED && \
                   (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH) )
                    RS232_rxAddressMode = addressMode;
                    RS232_rxAddressDetected = 0u;
                #endif /* End RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH*/
            #endif /* End RS232_CONTROL_REG_REMOVED */
        #else /* RS232_RXHW_ADDRESS_ENABLED */
            if(addressMode != 0u) { }     /* release compiler warning */
        #endif /* End RS232_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: RS232_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Set the first hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void RS232_SetRxAddress1(uint8 address) 

    {
        RS232_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: RS232_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Set the second hardware address compare value
    *
    * Parameters:
    *  address
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void RS232_SetRxAddress2(uint8 address) 
    {
        RS232_RXADDRESS2_REG = address;
    }

#endif  /* RS232_RX_ENABLED || RS232_HD_ENABLED*/


#if( (RS232_TX_ENABLED) || (RS232_HD_ENABLED) )

    #if(RS232_TX_INTERRUPT_ENABLED)

        /*******************************************************************************
        * Function Name: RS232_EnableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Enable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Enable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void RS232_EnableTxInt(void) 
        {
            CyIntEnable(RS232_TX_VECT_NUM);
        }


        /*******************************************************************************
        * Function Name: RS232_DisableTxInt
        ********************************************************************************
        *
        * Summary:
        *  Disable TX interrupt generation
        *
        * Parameters:
        *  None.
        *
        * Return:
        *  None.
        *
        * Theory:
        *  Disable the interrupt output -or- the interrupt component itself
        *
        *******************************************************************************/
        void RS232_DisableTxInt(void) 
        {
            CyIntDisable(RS232_TX_VECT_NUM);
        }

    #endif /* RS232_TX_INTERRUPT_ENABLED */


    /*******************************************************************************
    * Function Name: RS232_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configure which status bits trigger an interrupt event
    *
    * Parameters:
    *  intSrc: An or'd combination of the desired status bit masks (defined in
    *          the header file)
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void RS232_SetTxInterruptMode(uint8 intSrc) 
    {
        RS232_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: RS232_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Write a byte of data to the Transmit FIFO or TX buffer to be sent when the
    *  bus is available. WriteTxData sends a byte without checking for buffer room
    *  or status. It is up to the user to separately check status.
    *
    * Parameters:
    *  TXDataByte: byte of data to place in the transmit FIFO
    *
    * Return:
    * void
    *
    * Global Variables:
    *  RS232_txBuffer - RAM buffer pointer for save data for transmission
    *  RS232_txBufferWrite - cyclic index for write to txBuffer,
    *    incremented after each byte saved to buffer.
    *  RS232_txBufferRead - cyclic index for read from txBuffer,
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  RS232_initVar - checked to identify that the component has been
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void RS232_WriteTxData(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(RS232_initVar != 0u)
        {
            #if(RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH)

                /* Disable Tx interrupt. */
                /* Protect variables that could change on interrupt. */
                #if(RS232_TX_INTERRUPT_ENABLED)
                    RS232_DisableTxInt();
                #endif /* End RS232_TX_INTERRUPT_ENABLED */

                if( (RS232_txBufferRead == RS232_txBufferWrite) &&
                    ((RS232_TXSTATUS_REG & RS232_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    RS232_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(RS232_txBufferWrite >= RS232_TXBUFFERSIZE)
                    {
                        RS232_txBufferWrite = 0u;
                    }

                    RS232_txBuffer[RS232_txBufferWrite] = txDataByte;

                    /* Add to the software buffer. */
                    RS232_txBufferWrite++;

                }

                /* Enable Tx interrupt. */
                #if(RS232_TX_INTERRUPT_ENABLED)
                    RS232_EnableTxInt();
                #endif /* End RS232_TX_INTERRUPT_ENABLED */

            #else /* RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */

                /* Add directly to the FIFO. */
                RS232_TXDATA_REG = txDataByte;

            #endif /* End RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */
        }
    }


    /*******************************************************************************
    * Function Name: RS232_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Read the status register for the component
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the status register which is clear on read. It is up to
    *  the user to handle all bits in this return value accordingly, even if the bit
    *  was not enabled as an interrupt source the event happened and must be handled
    *  accordingly.
    *
    *******************************************************************************/
    uint8 RS232_ReadTxStatus(void) 
    {
        return(RS232_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: RS232_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Wait to send byte until TX register or buffer has room.
    *
    * Parameters:
    *  txDataByte: The 8-bit data value to send across the UART.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_txBuffer - RAM buffer pointer for save data for transmission
    *  RS232_txBufferWrite - cyclic index for write to txBuffer,
    *     checked to identify free space in txBuffer and incremented after each byte
    *     saved to buffer.
    *  RS232_txBufferRead - cyclic index for read from txBuffer,
    *     checked to identify free space in txBuffer.
    *  RS232_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void RS232_PutChar(uint8 txDataByte) 
    {
            #if(RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH)
                /* The temporary output pointer is used since it takes two instructions
                *  to increment with a wrap, and we can't risk doing that with the real
                *  pointer and getting an interrupt in between instructions.
                */
                uint8 loc_txBufferWrite;
                uint8 loc_txBufferRead;

                do{
                    /* Block if software buffer is full, so we don't overwrite. */
                    #if ((RS232_TXBUFFERSIZE > RS232_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Disable TX interrupt to protect variables that could change on interrupt */
                        CyIntDisable(RS232_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    loc_txBufferWrite = RS232_txBufferWrite;
                    loc_txBufferRead = RS232_txBufferRead;
                    #if ((RS232_TXBUFFERSIZE > RS232_MAX_BYTE_VALUE) && (CY_PSOC3))
                        /* Enable interrupt to continue transmission */
                        CyIntEnable(RS232_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }while( (loc_txBufferWrite < loc_txBufferRead) ? (loc_txBufferWrite == (loc_txBufferRead - 1u)) :
                                        ((loc_txBufferWrite - loc_txBufferRead) ==
                                        (uint8)(RS232_TXBUFFERSIZE - 1u)) );

                if( (loc_txBufferRead == loc_txBufferWrite) &&
                    ((RS232_TXSTATUS_REG & RS232_TX_STS_FIFO_FULL) == 0u) )
                {
                    /* Add directly to the FIFO. */
                    RS232_TXDATA_REG = txDataByte;
                }
                else
                {
                    if(loc_txBufferWrite >= RS232_TXBUFFERSIZE)
                    {
                        loc_txBufferWrite = 0u;
                    }
                    /* Add to the software buffer. */
                    RS232_txBuffer[loc_txBufferWrite] = txDataByte;
                    loc_txBufferWrite++;

                    /* Finally, update the real output pointer */
                    #if ((RS232_TXBUFFERSIZE > RS232_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntDisable(RS232_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                    RS232_txBufferWrite = loc_txBufferWrite;
                    #if ((RS232_TXBUFFERSIZE > RS232_MAX_BYTE_VALUE) && (CY_PSOC3))
                        CyIntEnable(RS232_TX_VECT_NUM);
                    #endif /* End TXBUFFERSIZE > 255 */
                }

            #else /* RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */

                while((RS232_TXSTATUS_REG & RS232_TX_STS_FIFO_FULL) != 0u)
                {
                    ; /* Wait for room in the FIFO. */
                }

                /* Add directly to the FIFO. */
                RS232_TXDATA_REG = txDataByte;

            #endif /* End RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: RS232_PutString
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: char pointer to character string of Data to Send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  This function will block if there is not enough memory to place the whole
    *  string, it will block until the entire string has been written to the
    *  transmit buffer.
    *
    *******************************************************************************/
    void RS232_PutString(const char8 string[]) 
    {
        uint16 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(RS232_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent*/
            while(string[buf_index] != (char8)0)
            {
                RS232_PutChar((uint8)string[buf_index]);
                buf_index++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: RS232_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Write a Sequence of bytes on the Transmit line. Data comes from RAM or ROM.
    *
    * Parameters:
    *  string: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of Bytes to be transmitted.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void RS232_PutArray(const uint8 string[], uint8 byteCount)
                                                                    
    {
        uint8 buf_index = 0u;
        /* If not Initialized then skip this function*/
        if(RS232_initVar != 0u)
        {
            do
            {
                RS232_PutChar(string[buf_index]);
                buf_index++;
            }while(buf_index < byteCount);
        }
    }


    /*******************************************************************************
    * Function Name: RS232_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Write a character and then carriage return and line feed.
    *
    * Parameters:
    *  txDataByte: uint8 Character to send.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void RS232_PutCRLF(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(RS232_initVar != 0u)
        {
            RS232_PutChar(txDataByte);
            RS232_PutChar(0x0Du);
            RS232_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: RS232_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Determine the amount of space left in the TX buffer and return the count in
    *  bytes
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Integer count of the number of bytes left in the TX buffer
    *
    * Global Variables:
    *  RS232_txBufferWrite - used to calculate left space.
    *  RS232_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 RS232_GetTxBufferSize(void)
                                                            
    {
        uint8 size;

        #if(RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(RS232_TX_INTERRUPT_ENABLED)
                RS232_DisableTxInt();
            #endif /* End RS232_TX_INTERRUPT_ENABLED */

            if(RS232_txBufferRead == RS232_txBufferWrite)
            {
                size = 0u;
            }
            else if(RS232_txBufferRead < RS232_txBufferWrite)
            {
                size = (RS232_txBufferWrite - RS232_txBufferRead);
            }
            else
            {
                size = (RS232_TXBUFFERSIZE - RS232_txBufferRead) + RS232_txBufferWrite;
            }

            /* Enable Tx interrupt. */
            #if(RS232_TX_INTERRUPT_ENABLED)
                RS232_EnableTxInt();
            #endif /* End RS232_TX_INTERRUPT_ENABLED */

        #else /* RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */

            size = RS232_TXSTATUS_REG;

            /* Is the fifo is full. */
            if((size & RS232_TX_STS_FIFO_FULL) != 0u)
            {
                size = RS232_FIFO_LENGTH;
            }
            else if((size & RS232_TX_STS_FIFO_EMPTY) != 0u)
            {
                size = 0u;
            }
            else
            {
                /* We only know there is data in the fifo. */
                size = 1u;
            }

        #endif /* End RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */

        return(size);
    }


    /*******************************************************************************
    * Function Name: RS232_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the TX RAM buffer by setting the read and write pointers both to zero.
    *  Clears the hardware TX FIFO.  Any data present in the FIFO will not be sent.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_txBufferWrite - cleared to zero.
    *  RS232_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM buffer will be lost when overwritten.
    *
    *******************************************************************************/
    void RS232_ClearTxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* Enter critical section */
        enableInterrupts = CyEnterCriticalSection();
        /* clear the HW FIFO */
        RS232_TXDATA_AUX_CTL_REG |=  RS232_TX_FIFO_CLR;
        RS232_TXDATA_AUX_CTL_REG &= (uint8)~RS232_TX_FIFO_CLR;
        /* Exit critical section */
        CyExitCriticalSection(enableInterrupts);

        #if(RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH)

            /* Disable Tx interrupt. */
            /* Protect variables that could change on interrupt. */
            #if(RS232_TX_INTERRUPT_ENABLED)
                RS232_DisableTxInt();
            #endif /* End RS232_TX_INTERRUPT_ENABLED */

            RS232_txBufferRead = 0u;
            RS232_txBufferWrite = 0u;

            /* Enable Tx interrupt. */
            #if(RS232_TX_INTERRUPT_ENABLED)
                RS232_EnableTxInt();
            #endif /* End RS232_TX_INTERRUPT_ENABLED */

        #endif /* End RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH */
    }


    /*******************************************************************************
    * Function Name: RS232_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Write a Break command to the UART
    *
    * Parameters:
    *  uint8 retMode:  Wait mode,
    *   0 - Initialize registers for Break, sends the Break signal and return
    *       imediately.
    *   1 - Wait until Break sending is complete, reinitialize registers to normal
    *       transmission mode then return.
    *   2 - Reinitialize registers to normal transmission mode then return.
    *   3 - both steps: 0 and 1
    *       init registers for Break, send Break signal
    *       wait until Break sending is complete, reinit registers to normal
    *       transmission mode then return.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_initVar - checked to identify that the component has been
    *     initialized.
    *  tx_period - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  Trere are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Funcition will block CPU untill transmition
    *     complete.
    *  2) User may want to use bloking time if UART configured to the low speed
    *     operation
    *     Emample for this case:
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to init and use the interrupt for complete
    *     break operation.
    *     Example for this case:
    *     Init TX interrupt whith "TX - On TX Complete" parameter
    *     SendBreak(0);     - init Break signal transmition
    *         Add your code here to use CPU time
    *     When interrupt appear with UART_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *   Uses static variable to keep registers configuration.
    *
    *******************************************************************************/
    void RS232_SendBreak(uint8 retMode) 
    {

        /* If not Initialized then skip this function*/
        if(RS232_initVar != 0u)
        {
            /*Set the Counter to 13-bits and transmit a 00 byte*/
            /*When that is done then reset the counter value back*/
            uint8 tmpStat;

            #if(RS232_HD_ENABLED) /* Half Duplex mode*/

                if( (retMode == RS232_SEND_BREAK) ||
                    (retMode == RS232_SEND_WAIT_REINIT ) )
                {
                    /* CTRL_HD_SEND_BREAK - sends break bits in HD mode*/
                    RS232_WriteControlRegister(RS232_ReadControlRegister() |
                                                          RS232_CTRL_HD_SEND_BREAK);
                    /* Send zeros*/
                    RS232_TXDATA_REG = 0u;

                    do /*wait until transmit starts*/
                    {
                        tmpStat = RS232_TXSTATUS_REG;
                    }while((tmpStat & RS232_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == RS232_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == RS232_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = RS232_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & RS232_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == RS232_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == RS232_REINIT) ||
                    (retMode == RS232_SEND_WAIT_REINIT) )
                {
                    RS232_WriteControlRegister(RS232_ReadControlRegister() &
                                                  (uint8)~RS232_CTRL_HD_SEND_BREAK);
                }

            #else /* RS232_HD_ENABLED Full Duplex mode */

                static uint8 tx_period;

                if( (retMode == RS232_SEND_BREAK) ||
                    (retMode == RS232_SEND_WAIT_REINIT) )
                {
                    /* CTRL_HD_SEND_BREAK - skip to send parity bit at Break signal in Full Duplex mode*/
                    #if( (RS232_PARITY_TYPE != RS232__B_UART__NONE_REVB) || \
                                        (RS232_PARITY_TYPE_SW != 0u) )
                        RS232_WriteControlRegister(RS232_ReadControlRegister() |
                                                              RS232_CTRL_HD_SEND_BREAK);
                    #endif /* End RS232_PARITY_TYPE != RS232__B_UART__NONE_REVB  */

                    #if(RS232_TXCLKGEN_DP)
                        tx_period = RS232_TXBITCLKTX_COMPLETE_REG;
                        RS232_TXBITCLKTX_COMPLETE_REG = RS232_TXBITCTR_BREAKBITS;
                    #else
                        tx_period = RS232_TXBITCTR_PERIOD_REG;
                        RS232_TXBITCTR_PERIOD_REG = RS232_TXBITCTR_BREAKBITS8X;
                    #endif /* End RS232_TXCLKGEN_DP */

                    /* Send zeros*/
                    RS232_TXDATA_REG = 0u;

                    do /* wait until transmit starts */
                    {
                        tmpStat = RS232_TXSTATUS_REG;
                    }while((tmpStat & RS232_TX_STS_FIFO_EMPTY) != 0u);
                }

                if( (retMode == RS232_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == RS232_SEND_WAIT_REINIT) )
                {
                    do /*wait until transmit complete*/
                    {
                        tmpStat = RS232_TXSTATUS_REG;
                    }while(((uint8)~tmpStat & RS232_TX_STS_COMPLETE) != 0u);
                }

                if( (retMode == RS232_WAIT_FOR_COMPLETE_REINIT) ||
                    (retMode == RS232_REINIT) ||
                    (retMode == RS232_SEND_WAIT_REINIT) )
                {

                    #if(RS232_TXCLKGEN_DP)
                        RS232_TXBITCLKTX_COMPLETE_REG = tx_period;
                    #else
                        RS232_TXBITCTR_PERIOD_REG = tx_period;
                    #endif /* End RS232_TXCLKGEN_DP */

                    #if( (RS232_PARITY_TYPE != RS232__B_UART__NONE_REVB) || \
                         (RS232_PARITY_TYPE_SW != 0u) )
                        RS232_WriteControlRegister(RS232_ReadControlRegister() &
                                                      (uint8)~RS232_CTRL_HD_SEND_BREAK);
                    #endif /* End RS232_PARITY_TYPE != NONE */
                }
            #endif    /* End RS232_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: RS232_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Set the transmit addressing mode
    *
    * Parameters:
    *  addressMode: 0 -> Space
    *               1 -> Mark
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void RS232_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable*/
        if(addressMode != 0u)
        {
            #if( RS232_CONTROL_REG_REMOVED == 0u )
                RS232_WriteControlRegister(RS232_ReadControlRegister() |
                                                      RS232_CTRL_MARK);
            #endif /* End RS232_CONTROL_REG_REMOVED == 0u */
        }
        else
        {
            #if( RS232_CONTROL_REG_REMOVED == 0u )
                RS232_WriteControlRegister(RS232_ReadControlRegister() &
                                                    (uint8)~RS232_CTRL_MARK);
            #endif /* End RS232_CONTROL_REG_REMOVED == 0u */
        }
    }

#endif  /* EndRS232_TX_ENABLED */

#if(RS232_HD_ENABLED)


    /*******************************************************************************
    * Function Name: RS232_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Rx configuration if required and loads the
    *  Tx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Tx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART.
    *
    * Side Effects:
    *  Disable RX interrupt mask, when software buffer has been used.
    *
    *******************************************************************************/
    void RS232_LoadTxConfig(void) 
    {
        #if((RS232_RX_INTERRUPT_ENABLED) && (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH))
            /* Disable RX interrupts before set TX configuration */
            RS232_SetRxInterruptMode(0u);
        #endif /* RS232_RX_INTERRUPT_ENABLED */

        RS232_WriteControlRegister(RS232_ReadControlRegister() | RS232_CTRL_HD_SEND);
        RS232_RXBITCTR_PERIOD_REG = RS232_HD_TXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(RS232_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */
    }


    /*******************************************************************************
    * Function Name: RS232_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Unloads the Tx configuration if required and loads the
    *  Rx configuration. It is the users responsibility to ensure that any
    *  transaction is complete and it is safe to unload the Rx
    *  configuration.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Valid only for half duplex UART
    *
    * Side Effects:
    *  Set RX interrupt mask based on customizer settings, when software buffer
    *  has been used.
    *
    *******************************************************************************/
    void RS232_LoadRxConfig(void) 
    {
        RS232_WriteControlRegister(RS232_ReadControlRegister() &
                                                (uint8)~RS232_CTRL_HD_SEND);
        RS232_RXBITCTR_PERIOD_REG = RS232_HD_RXBITCTR_INIT;
        #if(CY_UDB_V0) /* Manually clear status register when mode has been changed */
            /* Clear status register */
            CY_GET_REG8(RS232_RXSTATUS_PTR);
        #endif /* CY_UDB_V0 */

        #if((RS232_RX_INTERRUPT_ENABLED) && (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH))
            /* Enable RX interrupt after set RX configuration */
            RS232_SetRxInterruptMode(RS232_INIT_RX_INTERRUPTS_MASK);
        #endif /* RS232_RX_INTERRUPT_ENABLED */
    }

#endif  /* RS232_HD_ENABLED */


/* [] END OF FILE */
