/*******************************************************************************
* File Name: RS232_CRC_TX.c
* Version 2.40
*
* Description:
*  This file provides the source code to the API for the CRC component
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "RS232_CRC_TX.h"

uint8 RS232_CRC_TX_initVar = 0u;

RS232_CRC_TX_BACKUP_STRUCT RS232_CRC_TX_backup =
{
    0x00u, /* enableState; */

    /* Save D0 and D1 registers are none-retention for Panther ES1 */
    #if (CY_UDB_V0)

        #if(RS232_CRC_TX_CRC_SIZE <= 32u)
            RS232_CRC_TX_DEFAULT_POLYNOM, /* polynomial; */

        #else
            RS232_CRC_TX_DEFAULT_POLYNOM_UPPER, /* polynomialUpper; */
            RS232_CRC_TX_DEFAULT_POLYNOM_LOWER,  /* polynomialLower; */

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

    #endif  /* End (CY_UDB_V0) */
};


/*******************************************************************************
* Function Name: RS232_CRC_TX_Init
********************************************************************************
*
* Summary:
*  Initializes seed and polynomial registers with initial values.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void RS232_CRC_TX_Init(void) 
{
    /* Writes seed value and ponynom value provided for customizer */
    #if (RS232_CRC_TX_CRC_SIZE <= 32u)
        RS232_CRC_TX_WritePolynomial(RS232_CRC_TX_DEFAULT_POLYNOM);
        RS232_CRC_TX_WriteSeed(RS232_CRC_TX_DEFAULT_SEED);
        RS232_CRC_TX_ResetSeedInit(RS232_CRC_TX_DEFAULT_SEED);

    #else
        RS232_CRC_TX_WritePolynomialUpper(RS232_CRC_TX_DEFAULT_POLYNOM_UPPER);
        RS232_CRC_TX_WritePolynomialLower(RS232_CRC_TX_DEFAULT_POLYNOM_LOWER);
        RS232_CRC_TX_WriteSeedUpper(RS232_CRC_TX_DEFAULT_SEED_UPPER);
        RS232_CRC_TX_WriteSeedLower(RS232_CRC_TX_DEFAULT_SEED_LOWER);
        RS232_CRC_TX_ResetSeedInitUpper(RS232_CRC_TX_DEFAULT_SEED_UPPER);
        RS232_CRC_TX_ResetSeedInitLower(RS232_CRC_TX_DEFAULT_SEED_LOWER);

    #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
}


/*******************************************************************************
* Function Name: RS232_CRC_TX_Enable
********************************************************************************
*
* Summary:
*  Starts CRC computation on rising edge of input clock.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void RS232_CRC_TX_Enable(void) 
{
    RS232_CRC_TX_CONTROL_REG |= RS232_CRC_TX_CTRL_ENABLE;
}


/*******************************************************************************
* Function Name: RS232_CRC_TX_Start
********************************************************************************
*
* Summary:
*  Initializes seed and polynomial registers with initial values. Computation
*  of CRC starts on rising edge of input clock.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  RS232_CRC_TX_initVar: global variable is used to indicate initial
*  configuration of this component.  The variable is initialized to zero and set
*  to 1 the first time RS232_CRC_TX_Start() is called. This allows
*  enable/disable component without re-initialization in all subsequent calls
*  to the RS232_CRC_TX_Start() routine.
*
*******************************************************************************/
void RS232_CRC_TX_Start(void) 
{
    /* Writes seed value and ponynom value provided from customizer */
    if (RS232_CRC_TX_initVar == 0u)
    {
        RS232_CRC_TX_Init();
        RS232_CRC_TX_initVar = 1u;
    }

    RS232_CRC_TX_Enable();
}


/*******************************************************************************
* Function Name: RS232_CRC_TX_Stop
********************************************************************************
*
* Summary:
*  Stops CRC computation.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Reentrant:
*  No
*
*******************************************************************************/
void RS232_CRC_TX_Stop(void) 
{
    RS232_CRC_TX_CONTROL_REG &= ((uint8)~RS232_CRC_TX_CTRL_ENABLE);
}


#if (RS232_CRC_TX_CRC_SIZE <= 32u) /* 8-32 bits CRC */
    /*******************************************************************************
    * FUNCTION NAME: RS232_CRC_TX_ResetSeedInit
    ********************************************************************************
    *
    * Summary:
    *  Increments the CRC by one when API single step mode is used.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    void RS232_CRC_TX_ResetSeedInit(uint16 seed)
                                        
    {
         uint8 enableInterrupts;

        /* Mask the Seed to cut unused bits */
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            seed &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */

        /* Change AuxControl reg, need to be safety */
        enableInterrupts = CyEnterCriticalSection();

        #if (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE)
            /* Set FIFOs to single register */
            RS232_CRC_TX_AUX_CONTROL_A_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;

            #if(RS232_CRC_TX_CRC_SIZE > 16u)       /* 17-32 bits CRC */
                RS232_CRC_TX_AUX_CONTROL_B_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 8u) */

            /* AuxControl reg settings are done */
            CyExitCriticalSection(enableInterrupts);

            /* Write Seed COPY */
            #if (RS232_CRC_TX_CRC_SIZE <= 16u)          /* 16 bits CRC */
                RS232_CRC_TX_SEED_COPY_A__A1_REG = HI8(seed);
                RS232_CRC_TX_SEED_COPY_A__A0_REG = LO8(seed);

            #elif (RS232_CRC_TX_CRC_SIZE <= 24u)        /* 24 bits CRC */
                RS232_CRC_TX_SEED_COPY_B__A1_REG = LO8(HI16(seed));
                RS232_CRC_TX_SEED_COPY_B__A0_REG = HI8(seed);
                RS232_CRC_TX_SEED_COPY_A__A0_REG = LO8(seed);

            #else                                           /* 32 bits CRC */
                RS232_CRC_TX_SEED_COPY_B__A1_REG = HI8(HI16(seed));
                RS232_CRC_TX_SEED_COPY_A__A1_REG = LO8(HI16(seed));
                RS232_CRC_TX_SEED_COPY_B__A0_REG = HI8(seed);
                RS232_CRC_TX_SEED_COPY_A__A0_REG = LO8(seed);
            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        #else
            /* Set FIFOs to single register */
            #if (RS232_CRC_TX_CRC_SIZE <= 8u)         /* 8 bits CRC */
                RS232_CRC_TX_AUX_CONTROL_A_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;

            #elif (RS232_CRC_TX_CRC_SIZE <= 16u)      /* 16 bits CRC */
                RS232_CRC_TX_AUX_CONTROL_A_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
                RS232_CRC_TX_AUX_CONTROL_B_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;

            #elif (RS232_CRC_TX_CRC_SIZE <= 24u)      /* 24 bits CRC */
                RS232_CRC_TX_AUX_CONTROL_A_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
                RS232_CRC_TX_AUX_CONTROL_B_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
                RS232_CRC_TX_AUX_CONTROL_C_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;

            #elif (RS232_CRC_TX_CRC_SIZE <= 32u)      /* 32 bits CRC */
                RS232_CRC_TX_AUX_CONTROL_A_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
                RS232_CRC_TX_AUX_CONTROL_B_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
                RS232_CRC_TX_AUX_CONTROL_C_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
                RS232_CRC_TX_AUX_CONTROL_D_REG  |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;

            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 8u) */

            /* AuxControl reg setting are done */
            CyExitCriticalSection(enableInterrupts);

            /* Write Seed COPY */
            CY_SET_REG16(RS232_CRC_TX_SEED_COPY_PTR, seed);
        #endif  /* End (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE) */
    }

#else

    /*******************************************************************************
    * FUNCTION NAME: RS232_CRC_TX_ResetSeedInitUpper
    ********************************************************************************
    *
    * Summary:
    *  Increments the CRC by one when API single step mode is used.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    void RS232_CRC_TX_ResetSeedInitUpper(uint32 seed) 
    {
        uint8 enableInterrupts;

        /* Mask the Seed Upper half to cut unused bits in case, when CRC size is not byte-aligned or equal 24 */
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            seed &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */

        /* Change AuxControl reg, need to be safety */
        enableInterrupts = CyEnterCriticalSection();

        /* Set FIFOs to single register */
        RS232_CRC_TX_AUX_CONTROL_A_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
        RS232_CRC_TX_AUX_CONTROL_B_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
        RS232_CRC_TX_AUX_CONTROL_C_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;

        #if (RS232_CRC_TX_CRC_SIZE > 48u)               /* 49-64 bits CRC */
            RS232_CRC_TX_AUX_CONTROL_D_REG |= RS232_CRC_TX_AUXCTRL_FIFO_SINGLE_REG;
        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 8u) */

        /* AuxControl reg settings are done */
        CyExitCriticalSection(enableInterrupts);

        /* Write Seed Upper COPY */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            RS232_CRC_TX_SEED_UPPER_COPY_C__A1_REG = LO8(seed);

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            RS232_CRC_TX_SEED_UPPER_COPY_C__A1_REG = HI8(seed);
            RS232_CRC_TX_SEED_UPPER_COPY_B__A1_REG = LO8(seed);

        #elif (RS232_CRC_TX_CRC_SIZE <= 56u)        /* 56 bits CRC */
            RS232_CRC_TX_SEED_UPPER_COPY_D__A1_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_UPPER_COPY_C__A1_REG = HI8(seed);
            RS232_CRC_TX_SEED_UPPER_COPY_B__A1_REG = HI8(seed);

        #else                                           /* 64 bits CRC */
            RS232_CRC_TX_SEED_UPPER_COPY_D__A1_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_UPPER_COPY_C__A1_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_UPPER_COPY_B__A1_REG = HI8(seed);
            RS232_CRC_TX_SEED_UPPER_COPY_A__A1_REG = LO8(seed);

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
    }


    /*******************************************************************************
    * FUNCTION NAME: RS232_CRC_TX_ResetSeedInitLower
    ********************************************************************************
    *
    * Summary:
    *  Increments the CRC by one when API single step mode is used.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    void RS232_CRC_TX_ResetSeedInitLower(uint32 seed) 
    {
        /* Write Seed Lower COPY */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            RS232_CRC_TX_SEED_LOWER_COPY_B__A1_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_COPY_C__A0_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_COPY_B__A0_REG = HI8(seed);
            RS232_CRC_TX_SEED_LOWER_COPY_A__A0_REG = LO8(seed);

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            RS232_CRC_TX_SEED_LOWER_COPY_A__A1_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_COPY_C__A0_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_COPY_B__A0_REG = HI8(seed);
            RS232_CRC_TX_SEED_LOWER_COPY_A__A0_REG = LO8(seed);

        #else                                           /* 64 bits CRC */
            RS232_CRC_TX_SEED_LOWER_COPY_D__A0_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_COPY_C__A0_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_COPY_B__A0_REG = HI8(seed);
            RS232_CRC_TX_SEED_LOWER_COPY_A__A0_REG = LO8(seed);

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
    }
#endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */


#if(RS232_CRC_TX_CRC_SIZE <= 32u) /* 8-32 bits CRC */
    /*******************************************************************************
    * Function Name: RS232_CRC_TX_ReadCRC
    ********************************************************************************
    *
    * Summary:
    *  Reads CRC value.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  Returns CRC value.
    *
    * Side Effects:
    *  The CRC value is cut according to mask = 2^(Resolution) - 1.
    *  For example if CRC Resolution is 14 bits the mask value is:
    *  mask = 2^(14) - 1 = 0x3FFFu.
    *  The seed value = 0xFFFFu is cut:
    *  seed & mask = 0xFFFFu & 0x3FFFu = 0x3FFFu.
    *
    *******************************************************************************/
    uint16 RS232_CRC_TX_ReadCRC(void) 
    {
        /* Read CRC */
        #if (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE)

            uint16 seed;

            #if (RS232_CRC_TX_CRC_SIZE <= 16u)          /* 16 bits CRC */
                seed = ((uint16) RS232_CRC_TX_SEED_A__A1_REG) << 8u;
                seed |= RS232_CRC_TX_SEED_A__A0_REG;

            #elif (RS232_CRC_TX_CRC_SIZE <= 24u)        /* 24 bits CRC */
                seed = ((uint32) (RS232_CRC_TX_SEED_B__A1_REG)) << 16u;
                seed |= ((uint32) (RS232_CRC_TX_SEED_B__A0_REG)) << 8u;
                seed |= RS232_CRC_TX_SEED_A__A0_REG;

            #else                                           /* 32 bits CRC */
                seed = ((uint32) RS232_CRC_TX_SEED_B__A1_REG) << 24u;
                seed |= ((uint32) RS232_CRC_TX_SEED_A__A1_REG) << 16u;
                seed |= ((uint32) RS232_CRC_TX_SEED_B__A0_REG) << 8u;
                seed |= RS232_CRC_TX_SEED_A__A0_REG;

            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 8u) */
            
            #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
                seed &= RS232_CRC_TX_MASK;
            #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */
            
            return (seed);

        #else
            
            #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
                return (CY_GET_REG16(RS232_CRC_TX_SEED_PTR)  & RS232_CRC_TX_MASK);
            #else
                return (CY_GET_REG16(RS232_CRC_TX_SEED_PTR));
            #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */
            

        #endif  /* End (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE) */
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_WriteSeed
    ********************************************************************************
    *
    * Summary:
    *  Writes seed value.
    *
    * Parameters:
    *  seed:  Seed value.
    *
    * Return:
    *  void
    *
    * Side Effects:
    *  The seed value is cut according to mask = 2^(Resolution) - 1.
    *  For example if CRC Resolution is 14 bits the mask value is:
    *  mask = 2^(14) - 1 = 0x3FFFu.
    *  The seed value = 0xFFFFu is cut:
    *  seed & mask = 0xFFFFu & 0x3FFFu = 0x3FFFu.
    *
    *******************************************************************************/
    void RS232_CRC_TX_WriteSeed(uint16 seed) 
    {
        /* Mask the Seed Upper half to cut unused bits in case, when CRC size is not byte-aligned or equal 24 */
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            seed &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */

        /* Writes Seed */
        #if (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE)

            #if (RS232_CRC_TX_CRC_SIZE <= 16u)          /* 16 bits CRC */
                RS232_CRC_TX_SEED_A__A1_REG = HI8(seed);
                RS232_CRC_TX_SEED_A__A0_REG = LO8(seed);

            #elif (RS232_CRC_TX_CRC_SIZE <= 24u)        /* 24 bits CRC */
                RS232_CRC_TX_SEED_B__A1_REG = LO8(HI16(seed));
                RS232_CRC_TX_SEED_B__A0_REG = HI8(seed);
                RS232_CRC_TX_SEED_A__A0_REG = LO8(seed);

            #else                                           /* 32 bits CRC */
                RS232_CRC_TX_SEED_B__A1_REG = HI8(HI16(seed));
                RS232_CRC_TX_SEED_A__A1_REG = LO8(HI16(seed));
                RS232_CRC_TX_SEED_B__A0_REG = HI8(seed);
                RS232_CRC_TX_SEED_A__A0_REG = LO8(seed);
            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

            /* Resets triggers */
            RS232_CRC_TX_EXECUTE_DFF_RESET;

        #else

            CY_SET_REG16(RS232_CRC_TX_SEED_PTR, seed);

        #endif  /* End (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE) */
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_ReadPolynomial
    ********************************************************************************
    *
    * Summary:
    *  Reads CRC polynomial value.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  Returns CRC polynomial value.
    *
    *******************************************************************************/
    uint16 RS232_CRC_TX_ReadPolynomial(void)
                                               
    {
        /* Reads polynomial */
        #if (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE)

            uint16 polynomial;

            #if (RS232_CRC_TX_CRC_SIZE <= 16u)          /* 16 bits CRC */
                polynomial = ((uint16) RS232_CRC_TX_POLYNOM_A__D1_REG) << 8u;
                polynomial |= (RS232_CRC_TX_POLYNOM_A__D0_REG);

            #elif (RS232_CRC_TX_CRC_SIZE <= 24u)        /* 24 bits CRC */
                polynomial = ((uint32) RS232_CRC_TX_POLYNOM_B__D1_REG) << 16u;
                polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_B__D0_REG) << 8u;
                polynomial |= RS232_CRC_TX_POLYNOM_A__D0_REG;

            #else                                           /* 32 bits CRC */
                polynomial = ((uint32) RS232_CRC_TX_POLYNOM_B__D1_REG) << 24u;
                polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_A__D1_REG) << 16u;
                polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_B__D0_REG) << 8u;
                polynomial |= RS232_CRC_TX_POLYNOM_A__D0_REG;

            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

            return polynomial;

        #else

            return CY_GET_REG16(RS232_CRC_TX_POLYNOM_PTR);

        #endif  /* End (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE) */
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_WritePolynomial
    ********************************************************************************
    *
    * Summary:
    *  Writes CRC polynomial value.
    *
    * Parameters:
    *  polynomial:  CRC polynomial.
    *
    * Return:
    *  void
    *
    * Side Effects:
    *  The polynomial value is cut according to mask = 2^(Resolution) - 1.
    *  For example if CRC Resolution is 14 bits the mask value is:
    *  mask = 2^(14) - 1 = 0x3FFFu.
    *  The polynomial value = 0xFFFFu is cut:
    *  polynomial & mask = 0xFFFFu & 0x3FFFu = 0x3FFFu.
    *
    *******************************************************************************/
    void RS232_CRC_TX_WritePolynomial(uint16 polynomial)
                                          
    {
        /* Mask polynomial to cut unused bits in case, when CRC size is not byte-aligned or equal 24 */
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            polynomial &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */

        /* Write polynomial */
        #if (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE)

            #if (RS232_CRC_TX_CRC_SIZE <= 16u)          /* 16 bits CRC */

                RS232_CRC_TX_POLYNOM_A__D1_REG = HI8(polynomial);
                RS232_CRC_TX_POLYNOM_A__D0_REG = LO8(polynomial);

            #elif (RS232_CRC_TX_CRC_SIZE <= 24u)        /* 24 bits CRC */
                RS232_CRC_TX_POLYNOM_B__D1_REG = LO8(HI16(polynomial));
                RS232_CRC_TX_POLYNOM_B__D0_REG = HI8(polynomial);
                RS232_CRC_TX_POLYNOM_A__D0_REG = LO8(polynomial);

            #else                                           /* 32 bits CRC */
                RS232_CRC_TX_POLYNOM_B__D1_REG = HI8(HI16(polynomial));
                RS232_CRC_TX_POLYNOM_A__D1_REG = LO8(HI16(polynomial));
                RS232_CRC_TX_POLYNOM_B__D0_REG = HI8(polynomial);
                RS232_CRC_TX_POLYNOM_A__D0_REG = LO8(polynomial);

            #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

            /* Resets triggers */
            RS232_CRC_TX_EXECUTE_DFF_RESET;

        #else

            CY_SET_REG16(RS232_CRC_TX_POLYNOM_PTR, polynomial);

        #endif  /* End (RS232_CRC_TX_TIME_MULTIPLEXING_ENABLE) */
    }

#else   /* 33-64 bits CRC */

    /*******************************************************************************
    *  Function Name: RS232_CRC_TX_ReadCRCUpper
    ********************************************************************************
    *
    * Summary:
    *  Reads upper half of CRC value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  Returns upper half of CRC value.
    *
    * Side Effects:
    *  The polynomial value is cut according to mask = 2^(Resolution) - 1.
    *  For example if CRC Resolution is 14 bits the mask value is:
    *  mask = 2^(14) - 1 = 0x3FFFu.
    *  The polynomial value = 0xFFFFu is cut:
    *  polynomial & mask = 0xFFFFu & 0x3FFFu = 0x3FFFu.
    *
    *******************************************************************************/
    uint32 RS232_CRC_TX_ReadCRCUpper(void) 
    {
        uint32 seed;

        /* Read CRC Upper */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            seed = RS232_CRC_TX_SEED_UPPER_C__A1_REG;

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            seed = ((uint32) RS232_CRC_TX_SEED_UPPER_C__A1_REG) << 8u;
            seed |= RS232_CRC_TX_SEED_UPPER_B__A1_REG;

        #elif (RS232_CRC_TX_CRC_SIZE <= 56u)        /* 56 bits CRC */
            seed = ((uint32) RS232_CRC_TX_SEED_UPPER_D__A1_REG) << 16u;
            seed |= ((uint32) RS232_CRC_TX_SEED_UPPER_C__A1_REG) << 8u;
            seed |= RS232_CRC_TX_SEED_UPPER_B__A1_REG;

        #else                                           /* 64 bits CRC */
            seed = ((uint32) RS232_CRC_TX_SEED_UPPER_D__A1_REG) << 24u;
            seed |= ((uint32) RS232_CRC_TX_SEED_UPPER_C__A1_REG) << 16u;
            seed |= ((uint32) RS232_CRC_TX_SEED_UPPER_B__A1_REG) << 8u;
            seed |= RS232_CRC_TX_SEED_UPPER_A__A1_REG;

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
        
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            seed &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */
                
        return (seed);
    }


    /*******************************************************************************
    *  Function Name: RS232_CRC_TX_ReadCRCLower
    ********************************************************************************
    *
    * Summary:
    *  Reads lower half of CRC value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  Returns lower half of CRC value.
    *
    *******************************************************************************/
    uint32 RS232_CRC_TX_ReadCRCLower(void) 
    {
        uint32 seed;

        /* Read CRC Lower */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            seed = ((uint32) RS232_CRC_TX_SEED_LOWER_B__A1_REG) << 24u;
            seed |= ((uint32) RS232_CRC_TX_SEED_LOWER_C__A0_REG) << 16u;
            seed |= ((uint32) RS232_CRC_TX_SEED_LOWER_B__A0_REG) << 8u;
            seed |= RS232_CRC_TX_SEED_LOWER_A__A0_REG;

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            seed = ((uint32) RS232_CRC_TX_SEED_LOWER_A__A1_REG) << 24u;
            seed |= ((uint32) RS232_CRC_TX_SEED_LOWER_C__A0_REG) << 16u;
            seed |= ((uint32) RS232_CRC_TX_SEED_LOWER_B__A0_REG) << 8u;
            seed |= RS232_CRC_TX_SEED_LOWER_A__A0_REG;

        #else                                           /* 64 bits CRC */
            seed = ((uint32) RS232_CRC_TX_SEED_LOWER_D__A0_REG) << 24u;
            seed |= ((uint32) RS232_CRC_TX_SEED_LOWER_C__A0_REG) << 16u;
            seed |= ((uint32) RS232_CRC_TX_SEED_LOWER_B__A0_REG) << 8u;
            seed |= RS232_CRC_TX_SEED_LOWER_A__A0_REG;

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        return seed;
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_WriteSeedUpper
    ********************************************************************************
    *
    * Summary:
    *  Writes upper half of seed value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  seed:  Upper half of seed value.
    *
    * Return:
    *  void
    *
    * Side Effects:
    *  The upper half of seed value is cut according to
    *  mask = 2^(Resolution - 32) - 1.
    *  For example if CRC Resolution is 35 bits the mask value is:
    *  2^(35 - 32) - 1 = 2^(3) - 1 = 0x0000 0007u.
    *  The upper half of seed value = 0x0000 00FFu is cut:
    *  upper half of seed & mask = 0x0000 00FFu & 0x0000 0007u = 0x0000 0007u.
    *
    *******************************************************************************/
    void RS232_CRC_TX_WriteSeedUpper(uint32 seed) 
    {
        /* Mask the Seed Upper half to cut unused bits in case, when CRC size is not byte-aligned or equal 24 */
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            seed &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */

        /* Write Seed Upper */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            RS232_CRC_TX_SEED_UPPER_C__A1_REG = LO8(seed);

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            RS232_CRC_TX_SEED_UPPER_C__A1_REG = HI8(seed);
            RS232_CRC_TX_SEED_UPPER_B__A1_REG = LO8(seed);

        #elif (RS232_CRC_TX_CRC_SIZE <= 56u)        /* 56 bits CRC */
            RS232_CRC_TX_SEED_UPPER_D__A1_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_UPPER_C__A1_REG = HI8(seed);
            RS232_CRC_TX_SEED_UPPER_B__A1_REG = HI8(seed);

        #else                                           /* 64 bits CRC */
            RS232_CRC_TX_SEED_UPPER_D__A1_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_UPPER_C__A1_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_UPPER_B__A1_REG = HI8(seed);
            RS232_CRC_TX_SEED_UPPER_A__A1_REG = LO8(seed);

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        /* Resets triggers */
        RS232_CRC_TX_EXECUTE_DFF_RESET;
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_WriteSeedLower
    ********************************************************************************
    *
    * Summary:
    *  Writes lower half of seed value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  seed:  Lower half of seed value.
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    void RS232_CRC_TX_WriteSeedLower(uint32 seed) 
    {
        /* Write Seed Lower */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            RS232_CRC_TX_SEED_LOWER_B__A1_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_C__A0_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_B__A0_REG = HI8(seed);
            RS232_CRC_TX_SEED_LOWER_A__A0_REG = LO8(seed);

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            RS232_CRC_TX_SEED_LOWER_A__A1_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_C__A0_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_B__A0_REG = HI8(seed);
            RS232_CRC_TX_SEED_LOWER_A__A0_REG = LO8(seed);

        #else                                           /* 64 bits CRC */
            RS232_CRC_TX_SEED_LOWER_D__A0_REG = HI8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_C__A0_REG = LO8(HI16(seed));
            RS232_CRC_TX_SEED_LOWER_B__A0_REG = HI8(seed);
            RS232_CRC_TX_SEED_LOWER_A__A0_REG = LO8(seed);

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        /* Resets triggers */
        RS232_CRC_TX_EXECUTE_DFF_RESET;
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_ReadPolynomialUpper
    ********************************************************************************
    *
    * Summary:
    *  Reads upper half of CRC polynomial value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  Returns upper half of CRC polynomial value.
    *
    *******************************************************************************/
    uint32 RS232_CRC_TX_ReadPolynomialUpper(void) 
    {
        uint32 polynomial;

        /* Read Polynomial Upper */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            polynomial = RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG;

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            polynomial =  ((uint32) RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG) << 8u;
            polynomial |= RS232_CRC_TX_POLYNOM_UPPER_B__D1_REG;

        #elif (RS232_CRC_TX_CRC_SIZE <= 56u)        /* 56 bits CRC */
            polynomial =  ((uint32) RS232_CRC_TX_POLYNOM_UPPER_D__D1_REG) << 16u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG) << 8u;
            polynomial |= RS232_CRC_TX_POLYNOM_UPPER_B__D1_REG;

        #else                                           /* 64 bits CRC */
            polynomial =  ((uint32) RS232_CRC_TX_POLYNOM_UPPER_D__D1_REG) << 24u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG) << 16u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_UPPER_B__D1_REG) << 8u;
            polynomial |= RS232_CRC_TX_POLYNOM_UPPER_A__D1_REG;

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        return polynomial;
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_ReadPolynomialLower
    ********************************************************************************
    *
    * Summary:
    *  Reads lower half of CRC polynomial value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  void
    *
    * Return:
    *  Returns lower half of CRC polynomial value.
    *
    *******************************************************************************/
    uint32 RS232_CRC_TX_ReadPolynomialLower(void) 
    {
        uint32 polynomial;

        /* Read Polynomial Lower */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            polynomial =  ((uint32) RS232_CRC_TX_POLYNOM_LOWER_B__D1_REG) << 24u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_LOWER_C__D0_REG) << 16u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_LOWER_B__D0_REG) << 8u;
            polynomial |= RS232_CRC_TX_POLYNOM_LOWER_A__D0_REG;

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            polynomial =  ((uint32) RS232_CRC_TX_POLYNOM_LOWER_A__D1_REG) << 24u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_LOWER_C__D0_REG) << 16u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_LOWER_B__D0_REG) << 8u;
            polynomial |= RS232_CRC_TX_POLYNOM_LOWER_A__D0_REG;

        #else                                           /* 64 bits CRC */
            polynomial =  ((uint32) RS232_CRC_TX_POLYNOM_LOWER_D__D0_REG) << 24u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_LOWER_C__D0_REG) << 16u;
            polynomial |= ((uint32) RS232_CRC_TX_POLYNOM_LOWER_B__D0_REG) << 8u;
            polynomial |= RS232_CRC_TX_POLYNOM_LOWER_A__D0_REG;

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        return polynomial;
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_WritePolynomialUpper
    ********************************************************************************
    *
    * Summary:
    *  Writes upper half of CRC polynomial value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  polynomial:  Upper half CRC polynomial value.
    *
    * Return:
    *  void
    *
    * Side Effects:
    *  The upper half of polynomial value is cut according to
    *  mask = 2^(Resolution - 32) - 1.
    *  For example if CRC Resolution is 35 bits the mask value is:
    *  2^(35 - 32) - 1 = 2^(3) - 1 = 0x0000 0007u.
    *  The upper half of polynomial value = 0x0000 00FFu is cut:
    *  upper half of polynomial & mask = 0x0000 00FFu & 0x0000 0007u = 0x0000 0007u.
    *
    *******************************************************************************/
    void RS232_CRC_TX_WritePolynomialUpper(uint32 polynomial)
                                                
    {
        /* Mask the polynomial upper half to cut unused bits in case, when CRC size is not byte-aligned or equal 24 */
        #if((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u))
            polynomial &= RS232_CRC_TX_MASK;
        #endif /* ((0u != (RS232_CRC_TX_CRC_SIZE % 8u)) || (RS232_CRC_TX_CRC_SIZE == 24u)) */

        /* Write Polynomial Upper */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG = LO8(polynomial);

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG = HI8(polynomial);
            RS232_CRC_TX_POLYNOM_UPPER_B__D1_REG = LO8(polynomial);

        #elif (RS232_CRC_TX_CRC_SIZE <= 56u)        /* 56 bits CRC */
            RS232_CRC_TX_POLYNOM_UPPER_D__D1_REG = LO8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG = HI8(polynomial);
            RS232_CRC_TX_POLYNOM_UPPER_B__D1_REG = LO8(polynomial);

        #else                                           /* 64 bits CRC */
            RS232_CRC_TX_POLYNOM_UPPER_D__D1_REG = HI8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_UPPER_C__D1_REG = LO8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_UPPER_B__D1_REG = HI8(polynomial);
            RS232_CRC_TX_POLYNOM_UPPER_A__D1_REG = LO8(polynomial);

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        /* Resets triggers */
        RS232_CRC_TX_EXECUTE_DFF_RESET;
    }


    /*******************************************************************************
    * Function Name: RS232_CRC_TX_WritePolynomialLower
    ********************************************************************************
    *
    * Summary:
    *  Writes lower half of CRC polynomial value. Only generated for 33-64-bit CRC.
    *
    * Parameters:
    *  polynomial:  Lower half of CRC polynomial value.
    *
    * Return:
    *  void
    *
    *******************************************************************************/
    void RS232_CRC_TX_WritePolynomialLower(uint32 polynomial)
                                                
    {
        /* Write Polynomial Lower */
        #if (RS232_CRC_TX_CRC_SIZE <= 40u)          /* 40 bits CRC */
            RS232_CRC_TX_POLYNOM_LOWER_B__D1_REG = HI8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_LOWER_C__D0_REG = LO8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_LOWER_B__D0_REG = HI8(polynomial);
            RS232_CRC_TX_POLYNOM_LOWER_A__D0_REG = LO8(polynomial);

        #elif (RS232_CRC_TX_CRC_SIZE <= 48u)        /* 48 bits CRC */
            RS232_CRC_TX_POLYNOM_LOWER_A__D1_REG = HI8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_LOWER_C__D0_REG = LO8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_LOWER_B__D0_REG = HI8(polynomial);
            RS232_CRC_TX_POLYNOM_LOWER_A__D0_REG = LO8(polynomial);

        #else                                           /* 64 bits CRC */
            RS232_CRC_TX_POLYNOM_LOWER_D__D0_REG = HI8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_LOWER_C__D0_REG = LO8(HI16(polynomial));
            RS232_CRC_TX_POLYNOM_LOWER_B__D0_REG = HI8(polynomial);
            RS232_CRC_TX_POLYNOM_LOWER_A__D0_REG = LO8(polynomial);

        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */

        /* Resets triggers */
        RS232_CRC_TX_EXECUTE_DFF_RESET;
    }
#endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */


/* [] END OF FILE */
