/*******************************************************************************
* File Name: RS232_CRC_TX_PM.c
* Version 2.40
*
* Description:
*  This file provides Sleep APIs for CRC component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "RS232_CRC_TX.h"


/*******************************************************************************
* Function Name: RS232_CRC_TX_SaveConfig
********************************************************************************
*
* Summary:
*  Saves seed and polynomial registers.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global Variables:
*  RS232_CRC_TX_backup - modified when non-retention registers are saved.
*
*******************************************************************************/
void RS232_CRC_TX_SaveConfig(void) 
{       
    /* Save D0 and D1 registers are none-retention for Panther ES1 */
    #if (CY_UDB_V0)
    
        #if(RS232_CRC_TX_CRC_SIZE <= 32u)
            RS232_CRC_TX_backup.polynomial = RS232_CRC_TX_ReadPolynomial();
            
        #else
            RS232_CRC_TX_backup.polynomialUpper = RS232_CRC_TX_ReadPolynomialUpper();
            RS232_CRC_TX_backup.polynomialLower = RS232_CRC_TX_ReadPolynomialLower();
            
        #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
        
    #endif  /* End (CY_UDB_V0) */
}


/*******************************************************************************
* Function Name: RS232_CRC_TX_Sleep
********************************************************************************
*
* Summary:
*  Stops CRC computation and saves CRC configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global Variables:
*  RS232_CRC_TX_backup - modified when non-retention registers are saved.
*
*******************************************************************************/
void RS232_CRC_TX_Sleep(void) 
{
    /* Store CRC enable state */
    if(0u != (RS232_CRC_TX_CONTROL_REG & RS232_CRC_TX_CTRL_ENABLE))
    {
        RS232_CRC_TX_backup.enableState = 1u;
        RS232_CRC_TX_Stop();
    }
    else
    {
        RS232_CRC_TX_backup.enableState = 0u;
    }
    
    RS232_CRC_TX_SaveConfig();
}


/*******************************************************************************
* Function Name: RS232_CRC_TX_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores seed and polynomial registers.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void RS232_CRC_TX_RestoreConfig(void) 
{      
    /* Restore D0 and D1 registers are none-retention for Panther ES1 */
    #if (CY_UDB_V0)
    
        #if(RS232_CRC_TX_CRC_SIZE <= 32u)
            RS232_CRC_TX_WritePolynomial(RS232_CRC_TX_backup.polynomial);
            
        #else
            RS232_CRC_TX_WritePolynomialUpper(RS232_CRC_TX_backup.polynomialUpper);
            RS232_CRC_TX_WritePolynomialLower(RS232_CRC_TX_backup.polynomialLower);
            
        #endif  /* End (CY_UDB_V0) */
        
    #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
    
    #if (RS232_CRC_TX_CRC_SIZE <= 32u)
        RS232_CRC_TX_WriteSeed(RS232_CRC_TX_DEFAULT_SEED);                        
    #else
        RS232_CRC_TX_WriteSeedUpper(RS232_CRC_TX_DEFAULT_SEED_UPPER);
        RS232_CRC_TX_WriteSeedLower(RS232_CRC_TX_DEFAULT_SEED_LOWER); 
    #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
    
    #if (RS232_CRC_TX_CRC_SIZE <= 32u)
        RS232_CRC_TX_ResetSeedInit(RS232_CRC_TX_DEFAULT_SEED);                        
    #else
        RS232_CRC_TX_ResetSeedInitUpper(RS232_CRC_TX_DEFAULT_SEED_UPPER);
        RS232_CRC_TX_ResetSeedInitLower(RS232_CRC_TX_DEFAULT_SEED_LOWER); 
    #endif  /* End (RS232_CRC_TX_CRC_SIZE <= 32u) */
}


/*******************************************************************************
* Function Name: RS232_CRC_TX_Wakeup
********************************************************************************
*
* Summary:
*  Restores CRC configuration and starts CRC computation. 
*
* Parameters:
*  void
*
* Return:
*  void
* 
*******************************************************************************/
void RS232_CRC_TX_Wakeup(void) 
{
    RS232_CRC_TX_RestoreConfig();
    
    /* Restore CRC enable state */
    if (RS232_CRC_TX_backup.enableState != 0u)
    {
        RS232_CRC_TX_Enable();
    }
}


/* [] END OF FILE */
