/*******************************************************************************
* File Name: RS232_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "RS232.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (RS232_RX_ENABLED || RS232_HD_ENABLED) && \
     (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: RS232_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_rxBuffer - RAM buffer pointer for save received data.
    *  RS232_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  RS232_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  RS232_rxBufferOverflow - software overflow flag. Set to one
    *     when RS232_rxBufferWrite index overtakes
    *     RS232_rxBufferRead index.
    *  RS232_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when RS232_rxBufferWrite is equal to
    *    RS232_rxBufferRead
    *  RS232_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  RS232_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(RS232_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START RS232_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = RS232_RXSTATUS_REG;

        if((readData & (RS232_RX_STS_BREAK | RS232_RX_STS_PAR_ERROR |
                        RS232_RX_STS_STOP_ERROR | RS232_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START RS232_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & RS232_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (RS232_RXHW_ADDRESS_ENABLED)
                if(RS232_rxAddressMode == (uint8)RS232__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & RS232_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & RS232_RX_STS_ADDR_MATCH) != 0u)
                        {
                            RS232_rxAddressDetected = 1u;
                        }
                        else
                        {
                            RS232_rxAddressDetected = 0u;
                        }
                    }

                    readData = RS232_RXDATA_REG;
                    if(RS232_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        RS232_rxBuffer[RS232_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    RS232_rxBuffer[RS232_rxBufferWrite] = RS232_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                RS232_rxBuffer[RS232_rxBufferWrite] = RS232_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(RS232_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    RS232_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                RS232_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(RS232_rxBufferWrite >= RS232_RXBUFFERSIZE)
                {
                    RS232_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(RS232_rxBufferWrite == RS232_rxBufferRead)
                {
                    RS232_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(RS232_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        RS232_RXSTATUS_MASK_REG  &= (uint8)~RS232_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(RS232_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End RS232_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = RS232_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START RS232_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End RS232_RX_ENABLED && (RS232_RXBUFFERSIZE > RS232_FIFO_LENGTH) */


#if(RS232_TX_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: RS232_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  RS232_txBuffer - RAM buffer pointer for transmit data from.
    *  RS232_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  RS232_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(RS232_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START RS232_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((RS232_txBufferRead != RS232_txBufferWrite) &&
             ((RS232_TXSTATUS_REG & RS232_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(RS232_txBufferRead >= RS232_TXBUFFERSIZE)
            {
                RS232_txBufferRead = 0u;
            }

            RS232_TXDATA_REG = RS232_txBuffer[RS232_txBufferRead];

            /* Set next pointer. */
            RS232_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START RS232_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End RS232_TX_ENABLED && (RS232_TXBUFFERSIZE > RS232_FIFO_LENGTH) */


/* [] END OF FILE */
