/*******************************************************************************
* File Name: RS232_PM.c
* Version 2.30
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "RS232.h"


/***************************************
* Local data allocation
***************************************/

static RS232_BACKUP_STRUCT  RS232_backup =
{
    /* enableState - disabled */
    0u,
};



/*******************************************************************************
* Function Name: RS232_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  RS232_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void RS232_SaveConfig(void)
{
    #if (CY_UDB_V0)

        #if(RS232_CONTROL_REG_REMOVED == 0u)
            RS232_backup.cr = RS232_CONTROL_REG;
        #endif /* End RS232_CONTROL_REG_REMOVED */

        #if( (RS232_RX_ENABLED) || (RS232_HD_ENABLED) )
            RS232_backup.rx_period = RS232_RXBITCTR_PERIOD_REG;
            RS232_backup.rx_mask = RS232_RXSTATUS_MASK_REG;
            #if (RS232_RXHW_ADDRESS_ENABLED)
                RS232_backup.rx_addr1 = RS232_RXADDRESS1_REG;
                RS232_backup.rx_addr2 = RS232_RXADDRESS2_REG;
            #endif /* End RS232_RXHW_ADDRESS_ENABLED */
        #endif /* End RS232_RX_ENABLED | RS232_HD_ENABLED*/

        #if(RS232_TX_ENABLED)
            #if(RS232_TXCLKGEN_DP)
                RS232_backup.tx_clk_ctr = RS232_TXBITCLKGEN_CTR_REG;
                RS232_backup.tx_clk_compl = RS232_TXBITCLKTX_COMPLETE_REG;
            #else
                RS232_backup.tx_period = RS232_TXBITCTR_PERIOD_REG;
            #endif /*End RS232_TXCLKGEN_DP */
            RS232_backup.tx_mask = RS232_TXSTATUS_MASK_REG;
        #endif /*End RS232_TX_ENABLED */


    #else /* CY_UDB_V1 */

        #if(RS232_CONTROL_REG_REMOVED == 0u)
            RS232_backup.cr = RS232_CONTROL_REG;
        #endif /* End RS232_CONTROL_REG_REMOVED */

    #endif  /* End CY_UDB_V0 */
}


/*******************************************************************************
* Function Name: RS232_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  RS232_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void RS232_RestoreConfig(void)
{

    #if (CY_UDB_V0)

        #if(RS232_CONTROL_REG_REMOVED == 0u)
            RS232_CONTROL_REG = RS232_backup.cr;
        #endif /* End RS232_CONTROL_REG_REMOVED */

        #if( (RS232_RX_ENABLED) || (RS232_HD_ENABLED) )
            RS232_RXBITCTR_PERIOD_REG = RS232_backup.rx_period;
            RS232_RXSTATUS_MASK_REG = RS232_backup.rx_mask;
            #if (RS232_RXHW_ADDRESS_ENABLED)
                RS232_RXADDRESS1_REG = RS232_backup.rx_addr1;
                RS232_RXADDRESS2_REG = RS232_backup.rx_addr2;
            #endif /* End RS232_RXHW_ADDRESS_ENABLED */
        #endif  /* End (RS232_RX_ENABLED) || (RS232_HD_ENABLED) */

        #if(RS232_TX_ENABLED)
            #if(RS232_TXCLKGEN_DP)
                RS232_TXBITCLKGEN_CTR_REG = RS232_backup.tx_clk_ctr;
                RS232_TXBITCLKTX_COMPLETE_REG = RS232_backup.tx_clk_compl;
            #else
                RS232_TXBITCTR_PERIOD_REG = RS232_backup.tx_period;
            #endif /*End RS232_TXCLKGEN_DP */
            RS232_TXSTATUS_MASK_REG = RS232_backup.tx_mask;
        #endif /*End RS232_TX_ENABLED */

    #else /* CY_UDB_V1 */

        #if(RS232_CONTROL_REG_REMOVED == 0u)
            RS232_CONTROL_REG = RS232_backup.cr;
        #endif /* End RS232_CONTROL_REG_REMOVED */

    #endif  /* End CY_UDB_V0 */
}


/*******************************************************************************
* Function Name: RS232_Sleep
********************************************************************************
*
* Summary:
*  Stops and saves the user configuration. Should be called
*  just prior to entering sleep.
*
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  RS232_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void RS232_Sleep(void)
{

    #if(RS232_RX_ENABLED || RS232_HD_ENABLED)
        if((RS232_RXSTATUS_ACTL_REG  & RS232_INT_ENABLE) != 0u)
        {
            RS232_backup.enableState = 1u;
        }
        else
        {
            RS232_backup.enableState = 0u;
        }
    #else
        if((RS232_TXSTATUS_ACTL_REG  & RS232_INT_ENABLE) !=0u)
        {
            RS232_backup.enableState = 1u;
        }
        else
        {
            RS232_backup.enableState = 0u;
        }
    #endif /* End RS232_RX_ENABLED || RS232_HD_ENABLED*/

    RS232_Stop();
    RS232_SaveConfig();
}


/*******************************************************************************
* Function Name: RS232_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called
*  just after awaking from sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  RS232_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void RS232_Wakeup(void)
{
    RS232_RestoreConfig();
    #if( (RS232_RX_ENABLED) || (RS232_HD_ENABLED) )
        RS232_ClearRxBuffer();
    #endif /* End (RS232_RX_ENABLED) || (RS232_HD_ENABLED) */
    #if(RS232_TX_ENABLED || RS232_HD_ENABLED)
        RS232_ClearTxBuffer();
    #endif /* End RS232_TX_ENABLED || RS232_HD_ENABLED */

    if(RS232_backup.enableState != 0u)
    {
        RS232_Enable();
    }
}


/* [] END OF FILE */
